//
// Open Service Platform
// Copyright (c) 2012 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

/**
 * @file		ChannelService.h
 * @brief		This is the header file of the Channel service daemon.
 */


#ifndef _CHANNEL_SERVICE_H_
#define _CHANNEL_SERVICE_H_

#include <FApp.h>
#include <FBase.h>
#include <FIo.h>


namespace Tizen { namespace Io {

class _ChannelService;
class _ChannelServiceStub;

} }

class _MessagePortService;
class _MessagePortStub;

/**
 * [ChannelService] ServiceApp must inherit from ServiceApp class
 * which provides basic features necessary to define an ServiceApp.
 */
class ChannelService
	: public Tizen::App::ServiceApp
{
public:

	/**
	 * [ChannelService] ServiceApp must have a factory method that creates an instance of itself.
	 */
	static Tizen::App::ServiceApp* CreateInstance(void);

public:

	ChannelService();
	~ChannelService();

public:

	// Called when the ServiceApp is initializing.
	bool OnAppInitializing(Tizen::App::AppRegistry& appRegistry);

	// Called when the ServiceApp initializing is finished.
	bool OnAppInitialized(void);

	// Called when the ServiceApp is requested to terminate.
	bool OnAppWillTerminate(void);

	// Called when the ServiceApp is terminating.
	bool OnAppTerminating(Tizen::App::AppRegistry& appRegistry, bool forcedTermination = false);

	// Called when the system memory is not sufficient to run the ServiceApp any further.
	void OnLowMemory(void);

private:
	Tizen::Io::_ChannelService*	__pChannelService;
	Tizen::Io::_ChannelServiceStub*	__pChannelServiceStub;

	_MessagePortService*	__pMessagePortService;
	_MessagePortStub*	__pMessagePortStub;
}; // ChannelService

#endif // _CHANNEL_SERVICE_H_
