//
// Open Service Platform
// Copyright (c) 2012 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

/**
 * @file	FIo_ChannelCAppStub.h
 * @brief	This is the header file for the _ChannelCAppStub class.
 *
 * This file contains the declarations of _ChannelCAppStub.
 */

#ifndef _FIO_INTERNAL_CHANNEL_CAPP_STUB_H_
#define _FIO_INTERNAL_CHANNEL_CAPP_STUB_H_

#include <string>
#include <pthread.h>
#include <map>
#include <glib.h>

#include <FOspConfig.h>
#include <FBaseObject.h>
#include <FBaseString.h>
#include <FBaseResult.h>
#include <FBaseColArrayList.h>
#include <FIo_ChannelService.h>

namespace Tizen { namespace Base { namespace Runtime
{
class _EventDispatcher;
}}}

namespace Tizen { namespace Io
{

class _IIpcServerEventListener;
class _ChannelServiceStub;

/**
 * @class _ChannelCAppStub
 * @brief This class provides methods to handle c-application request messages.
 * @since 2.1
 *
 */
class _OSP_EXPORT_ _ChannelCAppStub
	: public Tizen::Base::Object
{
public:
	/**
	 * This is the default constructor for this class.
	 * @since 2.1
	 */
	_ChannelCAppStub(void);

	/**
	 * This is the destructor for this class.
	 * @since 2.1
	 */
	virtual ~_ChannelCAppStub(void);

	result Construct(void);

	void SetChannelService(_ChannelService& service);

	static bool SendResponse(int requestId, void* pGIOChannel, const Tizen::Base::Collection::ArrayList& args);

private:
	_ChannelCAppStub(const _ChannelCAppStub& rhs);

	_ChannelCAppStub& operator =(const _ChannelCAppStub& rhs);

	static gboolean OnConnectionRequest(GIOChannel* source, GIOCondition condition, gpointer data);

	static gboolean OnReadMessage(GIOChannel* source, GIOCondition condition, gpointer data);

	gboolean HandleReceivedMessage(GIOChannel* source, GIOCondition condition, gpointer data);

	_ChannelService* __pChannelService;

	struct  _ClientInfo;

	struct  _ChannelInfo
	{
		_ChannelInfo(void);
		~_ChannelInfo(void);

		struct _ClientInfo* pClientInfo;
		GIOChannel* pGIOChannel;
		GSource* pGSource;
		bool destroySource;
	};

	struct  _ClientInfo
	{
		_ClientInfo(void);
		~_ClientInfo(void);

		int clientId;                              /**< the client id */
		_ChannelCAppStub* pChannelStub;                    /**< the pointer to an _ChannelCAppStub */
		std::vector <struct _ChannelInfo*> channels;   /**< the set of channels associated with a client */
		Tizen::Base::String appId;
		//Tizen::Base::String appExecutableName;
	};

	GMainContext* __pGMainContext;

	GSource* __pConnectGSource;

	std::map <int, _ClientInfo*> __clients;   // pid of client is used for key

}; // _ChannelCAppStub

}} // Tizen::Io

#endif // _FIO_INTERNAL_CHANNEL_CAPP_STUB_H_

