//
// Open Service Platform
// Copyright (c) 2012 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

/**
 * @file	FIo_MessagePortService.h
 * @brief	This is the header file for the _MessagePortService class.
 *
 * This file contains the declarations of _MessagePortService.
 */

#ifndef _FIO_INTERNAL_MESSAGE_PORT_SERVICE_H_
#define _FIO_INTERNAL_MESSAGE_PORT_SERVICE_H_

#include <message-port-data-types.h>

#include <FBaseResult.h>
#include <FBaseString.h>
#include <FBaseColHashMap.h>
#include <FBaseColHashMapT.h>
#include <FAppTypes.h>

namespace IPC
{
class Message;
}

class _MessagePortStub;

class _MessagePortService
{
public:
	_MessagePortService(void);

	virtual ~_MessagePortService(void);

	virtual result Construct(_MessagePortStub& stub);

	virtual int RegisterMessagePort(int clientId, const BundleBuffer& buffer);

	virtual int CheckRemotePort(const BundleBuffer& buffer);

	result UnregisterMessagePort(int clientId);

	virtual int SendMessage(const BundleBuffer& buffer);

private:
	Tizen::Base::String GetKey(const BundleBuffer& buffer, bool local = true) const;

	bool IsPreloaded(const Tizen::Base::String& localAppId, const Tizen::Base::String& remoteAppId) const;

	int CheckCertificate(const Tizen::Base::String& localAppId, const Tizen::Base::String& remoteAppId) const;

	class _StringHashProvider
		: public Tizen::Base::Collection::IHashCodeProviderT<Tizen::Base::String>
	{
	public:
		virtual int GetHashCode(const Tizen::Base::String& value) const
		{
			return value.GetHashCode();
		}
	};

	class _StringComparer
		: public Tizen::Base::Collection::IComparerT<Tizen::Base::String>
	{
	public:
		virtual result Compare(const Tizen::Base::String& string1, const Tizen::Base::String& string2, int& cmp) const
		{
			cmp = string1.CompareTo(string2);
			return E_SUCCESS;
		}
	};

private:
	_MessagePortStub* __pStub;

	Tizen::Base::Collection::HashMapT <Tizen::Base::String, int>* __pPorts;
	Tizen::Base::Collection::HashMapT <Tizen::Base::String, int>* __pTrustedPorts;
}; // _MessagePortService

#endif // _FIO_INTERNAL_MESSAGE_PORT_SERVICE_H_
