//
// Open Service Platform
// Copyright (c) 2012 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

/**
 * @file	 ChannelService.cpp
 * @brief	This is the implementation for the ChannelService class.
 */

#include "ChannelService.h"

#include <FBaseSysLog.h>
#include <FIo_ChannelService.h>
#include <FIo_ChannelServiceStub.h>
#include <FIo_ChannelServiceManager.h>

#include <FIo_MessagePortStub.h>
#include <FIo_MessagePortService.h>

using namespace Tizen::Base;
using namespace Tizen::Io;
using namespace Tizen::App;

ChannelService::ChannelService()
: __pChannelService(null)
, __pChannelServiceStub(null)
, __pMessagePortService(null)
, __pMessagePortStub(null)
{
	SysLog(NID_IO, "Enter.");
}

ChannelService::~ChannelService()
{
	SysLog(NID_IO, "Exit.");

	delete __pChannelService;
	delete __pChannelServiceStub;

	delete __pMessagePortService;
	delete __pMessagePortStub;
}

ServiceApp*
ChannelService::CreateInstance(void)
{
	// Create the instance through the constructor.
	return new ChannelService();
}

bool
ChannelService::OnAppInitializing(AppRegistry& appRegistry)
{
	result r = E_SUCCESS;

	__pChannelServiceStub = new (std::nothrow) _ChannelServiceStub();
	if (__pChannelServiceStub == null)
	{
		return false;
	}

	r = __pChannelServiceStub->Construct();
	if (IsFailed(r))
	{
		delete __pChannelServiceStub;
		__pChannelServiceStub = null;

		return false;
	}

	__pChannelService = new (std::nothrow) _ChannelService;
	if (__pChannelService == null)
	{
		delete __pChannelServiceStub;
		__pChannelServiceStub = null;

		return false;
	}

	r = __pChannelService->Construct(*__pChannelServiceStub);
	if (IsFailed(r))
	{
		delete __pChannelService;
		__pChannelService = null;

		delete __pChannelServiceStub;
		__pChannelServiceStub = null;

		return false;
	}

	_ChannelServiceManager::GetInstance()->SetChannelService(__pChannelService);


	// Message Port
	__pMessagePortStub = new (std::nothrow) _MessagePortStub();
	if (__pMessagePortStub == null)
	{
		return false;
	}

	r = __pMessagePortStub->Construct();
	if (IsFailed(r))
	{
		delete __pMessagePortStub;
		__pMessagePortStub = null;

		return false;
	}

	__pMessagePortService = new (std::nothrow) _MessagePortService();
	if (__pMessagePortService == null)
	{
		delete __pMessagePortStub;
		__pMessagePortStub = null;

		return false;
	}

	r = __pMessagePortService->Construct(*__pMessagePortStub);
	if (IsFailed(r))
	{
		delete __pMessagePortService;
		__pMessagePortService = null;

		delete __pMessagePortStub;
		__pMessagePortStub = null;

		return false;
	}

	__pMessagePortStub->SetMessagePortService(*__pMessagePortService);

	return true;
}

bool
ChannelService::OnAppInitialized(void)
{
	return true;
}

bool
ChannelService::OnAppWillTerminate(void)
{
	return true;
}

bool
ChannelService::OnAppTerminating(AppRegistry& appRegistry, bool forcedTermination)
{
	return true;
}

void
ChannelService::OnLowMemory(void)
{
}

